local *

{
  :AcceptQuest
  :C_GossipInfo
  :CompleteQuest
  :GetNumActiveQuests
  :GetNumAvailableQuests
  :GetNumQuestChoices
  :GetNumQuestLogEntries
  :GetQuestLogSelection
  :GetQuestLogTitle
  :IsAltKeyDown
  :IsControlKeyDown
  :IsQuestCompletable
  :IsShiftKeyDown
  :QuestFrameCompleteQuestButton
  :SelectActiveQuest
  :SelectAvailableQuest
  :SelectQuestLogEntry
  :GossipTitleButton1
  :GossipTitleButton2
  :ShowQuestComplete
  :UnitName
  :idTweaks
  :idTweaksModule
  :idTweaksOptions
} = _G
{
  :Option
  :SelectOption
  :ToggleOption
} = idTweaksOptions

class Questing extends idTweaksModule
  new: (addon) =>
    -- print "Questing#new"

    super addon, "Questing", "AceConsole-3.0", "AceEvent-3.0"
    
    @completed_quests = {}
    @incomplete_quests = {}

  set_options: =>
    -- print "Questing#set_options"

    super!

    @options\add_option SelectOption @, "pause_key", "Pause Key", "While you hold this key nothing will happen", {
      values:
        shift: "Shift"
        control: "Control"
        alt: "Alt"
    }, {}, "shift"

    @options\add_option ToggleOption @, "accept_quests", "Accept Quests", "Accept quests"
    @options\add_option ToggleOption @, "complete_quests", "Complete Quests", "Automatically complete quests unless when offered a choice of rewards"
    @options\add_option ToggleOption @, "complete_gossip", "Complete Gossip", "When the gossip window has one option, use it"
    -- @options\add_option ToggleOption @, "complete_covenant_sanctum", "Complete Seat of the Primus", "Automate interactions in the Seat of the Primus"

  enable: =>
    -- print "Questing#enable"

    @ace_module\RegisterEvent "GOSSIP_SHOW", () ->
      return unless @should_automate!
      @automate_gossip_window!

    @ace_module\RegisterEvent "QUEST_AUTOCOMPLETE", (_, quest_id) ->
      return unless @should_automate!
      return unless @should_complete!
      @autocomplete_quest quest_id

    @ace_module\RegisterEvent "QUEST_COMPLETE", () ->
      return unless @should_automate!
      return unless @should_complete!
      @complete_quest!

    @ace_module\RegisterEvent "QUEST_DETAIL", () ->
      return unless @should_automate!
      return unless @should_accept!
      @accept_quest!

    @ace_module\RegisterEvent "QUEST_GREETING", () ->
      return unless @should_automate!
      @automate_gossip_window!

    @ace_module\RegisterEvent "QUEST_LOG_UPDATE", () ->
      return unless idTweaks\is_bc!
      return unless @should_automate!
      @index_quests!
      SelectQuestLogEntry GetQuestLogSelection!

    @ace_module\RegisterEvent "QUEST_PROGRESS", () ->
      return unless @should_automate!
      return unless @should_complete!
      @progress_quest!

  disable: =>
    -- print "Questing#disable"

    @ace_module\UnregisterEvent "GOSSIP_SHOW"
    @ace_module\UnregisterEvent "QUEST_AUTOCOMPLETE"
    @ace_module\UnregisterEvent "QUEST_COMPLETE"
    @ace_module\UnregisterEvent "QUEST_DETAIL"
    @ace_module\UnregisterEvent "QUEST_GREETING"
    @ace_module\UnregisterEvent "QUEST_LOG_UPDATE"
    @ace_module\UnregisterEvent "QUEST_PROGRESS"

  should_automate: =>
    -- print "Questing#should_automate"

    output = true
    key = @db.pause_key

    if key == "shift"
      if IsShiftKeyDown!
        output = false

    if key == "control"
      if IsControlKeyDown!
        output = false

    if key == "alt"
      if IsAltKeyDown!
        output = false

    output

  should_accept: =>
    -- print "Questing#should_accept"

    return @db.accept_quests

  should_complete: =>
    -- print "Questing#should_complete"

    return @db.complete_quests

  strip_text: (text) =>
    -- print "Questing#strip_text"

    return unless text

    text = text\gsub "|c%x%x%x%x%x%x%x%x(.-)|r", "%1"
    text = text\gsub "%[.*%]%s*", ""
    text = text\gsub "(.+) %(.+%)", "%1"
    text = text\trim!

    text
    
  index_quests: =>
    -- print "Questing#index_quests"

    @completed_quests = {}
    @incomplete_quests = {}
    
    num_entries = GetNumQuestLogEntries!

    if num_entries > 1
      for i = 1, num_entries
        SelectQuestLogEntry i
        title, _, _, _, _, is_complete = GetQuestLogTitle i
        
        if title
          if is_complete
            @completed_quests[title] = true
          else
            @incomplete_quests[title] = true

  accept_quest: () =>
    -- print "Questing#accept_quest"

    AcceptQuest!

  autocomplete_quest: (quest_id) =>
    -- print "Questing#autocomplete_quest"

    ShowQuestComplete quest_id

  progress_quest: () =>
    -- print "Questing#progress_quest"

    return unless @should_complete!
    CompleteQuest! if IsQuestCompletable!

  complete_quest: () =>
    -- print "Questing#complete_quest"

    QuestFrameCompleteQuestButton\Click! if GetNumQuestChoices! <= 1

  automate_gossip_window: () =>
    -- print "Questing#auto_gossip_window"

    if idTweaks\is_bc!
      -- If there is one gossip option, use it.
      if GossipTitleButton1\IsVisible! and (GossipTitleButton1.type == "Gossip" or GossipTitleButton1.type == "Active") and not GossipTitleButton2\IsVisible!
        GossipTitleButton1\Click!

      -- Complete Quests
      for i = 32, 1, -1
        button = _G["QuestTitleButton#{i}"]
        if button and button\IsVisible!
          text = @strip_text button\GetText!
          if @completed_quests[text] and @should_complete!
            button\Click!
          elseif @incomplete_quests[text] and @should_accept!
            button\Click!

      -- Complete Quests (Gossip Button)
      for i = 32, 1, -1
        button = _G["GossipTitleButton#{i}"]
        if button and button\IsVisible!
          text = @strip_text button\GetText!
          if @completed_quests[text] and @should_complete!
            button\Click!
          elseif @incomplete_quests[text] and @should_accept!
            button\Click!

    if idTweaks\is_retail!
      -- In the new API, complete quests and then accept them. You always get all progress in one click.
      num_active_quests = C_GossipInfo.GetNumActiveQuests!
      if num_active_quests > 0
        for i = 1, num_active_quests
          C_GossipInfo.SelectActiveQuest i

      num_available_quests = C_GossipInfo.GetNumAvailableQuests!
      if num_available_quests > 0
        for i = 1, num_available_quests
          C_GossipInfo.SelectAvailableQuest i

      -- In the old API, accept quests before completing quests. You might have to click multiple times.
      num_available_quests = GetNumAvailableQuests!
      if num_available_quests > 0
        for i = 1, num_available_quests
          SelectAvailableQuest i

      num_active_quests = GetNumActiveQuests!
      if num_active_quests > 0
        for i = 1, num_active_quests
          SelectActiveQuest i

      num_gossips = C_GossipInfo.GetNumOptions!
      if num_gossips == 1
        C_GossipInfo.SelectOption 1

idTweaks\add_module Questing idTweaks
